#include <avr/io.h>
#include <avr/interrupt.h>

#define F_CPU 16000000L
#include <util/delay.h>

void inline led_on () {
	PORTD |= (1 << PD7);
}

void inline led_off () {
	PORTD &= ~(1 << PD7);
}

void init_pwm() {

	TCCR1A |= (1 << WGM11) | (1 << COM1B1) | (1 << COM1A1);
	TCCR1B |= (1 << WGM12) | (1 << WGM13) | (1 << CS12);
	DDRD |= (1 << PD4) | (1 << PD5);
	DDRD |= (1 << PD7);
	DDRD|= ( 1 << PD0 ) | ( 1 << PD1 );
	ICR1 = 1000;
}

void ADC_init(void)
{
	// enable ADC with:
	// *reference AVCC with external capacitor at AREF pin
	// *without left adjust of conversion result
	// *no auto-trigger
	// *no interrupt
	// *prescaler at 128
	ADMUX = (1 << REFS0);
	ADCSRA = (1 << ADEN) | (7 << ADPS0);
}

int ADC_get(uint8_t channel)
{
	// start ADC conversion on "channel"
	// wait for completion
	// return the result
	ADMUX = (ADMUX & ~(0x1f << MUX0)) | channel;

	ADCSRA |= (1 << ADSC);
	while(ADCSRA & (1 << ADSC));

	return ADC;
}


int main(void)
{
	/* Initializari */
	DDRA = 0;
	PORTA = 0xFF;
	PORTD = 0;
	
	init_pwm();
	ADC_init();

	OCR1B = 450; // dreapta
	OCR1A = 450; // stanga

	int val = 0, i;
	int last_val;
	int check;
	
	/* ponderea senzorilor */
	int x[8] = {-1, -1, -1, -1, 1, 1, 1, 1};

	while (1) {

		val = 0;
		check = 0;
		
		/* afla valoarea senzorilor */
		for (i = 0; i < 8; i++) {
			if (ADC_get(i) > 340) {
				val += x[i];
				check = 1;
				led_on();
			} else {
				led_off();
			}
		}
		
		if(check == 0) { // a iesit de pe linie	
			val = last_val;
		}
		/* dreapta */
		if (val < 0) {

			OCR1B = 500;	
			OCR1A = 0;

		} else {
			/* stanga */
			if (val > 0) {

				OCR1B = 0;
				OCR1A = 500;
			
			} else {
				/* inainte */
				OCR1B = 450;
				OCR1A = 450;
			}
		}
		
		last_val = val;
	}
	return 0;
}
